@@ -21,6 +21,10 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 
 #include "mount-plugin.h"
@@ -28,12 +32,6 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 void format_LVM_name (const char *disk_device, gchar **formatted_diskname);
 
-#ifdef LIBXFCE4PANEL_CHECK_VERSION
-#if LIBXFCE4PANEL_CHECK_VERSION (4,9,0)
-#define HAS_PANEL_49
-#endif
-#endif
-
 #define gtk_hbox_new(homogeneous, spacing) \
         gtk_box_new(GTK_ORIENTATION_HORIZONTAL, spacing)
 
@@ -69,11 +67,19 @@ on_activate_disk_display (GtkWidget *widget, t_disk * disk)
 static void
 mounter_set_size (XfcePanelPlugin *plugin, int size, t_mounter *mt)
 {
-   /* shrink the gtk button's image to new size -*/
-#ifdef HAS_PANEL_49
-   size /= xfce_panel_plugin_get_nrows (plugin);
+#if LIBXFCE4PANEL_CHECK_VERSION (4, 17, 4)
+    xfce_panel_set_image_from_source (GTK_IMAGE (mt->image), mt->icon, NULL,
+                                      xfce_panel_plugin_get_icon_size (plugin),
+                                      gtk_widget_get_scale_factor (GTK_WIDGET (plugin)));
+#else
+    gint scale_factor = gtk_widget_get_scale_factor (GTK_WIDGET (plugin));
+    gint icon_size = xfce_panel_plugin_get_icon_size (plugin);
+    GdkPixbuf *pixbuf = xfce_panel_pixbuf_from_source (mt->icon, NULL, icon_size * scale_factor);
+    cairo_surface_t *surface = gdk_cairo_surface_create_from_pixbuf (pixbuf, scale_factor, NULL);
+    gtk_image_set_from_surface (GTK_IMAGE (mt->image), surface);
+    cairo_surface_destroy (surface);
+    g_object_unref (pixbuf);
 #endif
-   gtk_widget_set_size_request (GTK_WIDGET(mt->button), size, size);
 }
 
 /**
@@ -111,24 +117,39 @@ format_LVM_name (const char *disk_device, gchar **formatted_diskname)
 static void
 disk_display_set_sizes (GPtrArray *array)
 {
-    unsigned int i, max_width_label_disk=0, max_width_label_mount_info=0, tmp;
+    unsigned int i, max_width_label_disk=0, max_width_label_mount_point=0, max_width_label_mount_info=0, tmp;
     t_disk_display *disk_display;
 
     for (i=0; i<array->len; i++) {
         disk_display= g_ptr_array_index (array, i); /* get the disk */
+
         tmp = strlen(gtk_label_get_text(GTK_LABEL(disk_display->label_mount_info)));
         if (tmp>max_width_label_mount_info)
             max_width_label_mount_info = tmp;
 
-        tmp = strlen(gtk_label_get_text(GTK_LABEL(disk_display->label_disk)));
-        if (tmp>max_width_label_disk)
-            max_width_label_disk = tmp;
+        tmp = strlen(gtk_label_get_text(GTK_LABEL(disk_display->label_mount_point)));
+        if (tmp>max_width_label_mount_point)
+            max_width_label_mount_point = tmp;
+
+        if (disk_display->label_disk != NULL)
+        {
+            tmp = strlen(gtk_label_get_text(GTK_LABEL(disk_display->label_disk)));
+            if (tmp>max_width_label_disk)
+                max_width_label_disk = tmp;
+        }
     }
 
     for (i=0; i<array->len; i++) {
         disk_display = g_ptr_array_index (array, i); /* get the disk */
-        gtk_label_set_width_chars(GTK_LABEL(disk_display->label_disk), max_width_label_disk);
+
         gtk_label_set_width_chars(GTK_LABEL(disk_display->label_mount_info), max_width_label_mount_info);
+
+        gtk_label_set_width_chars(GTK_LABEL(disk_display->label_mount_point), max_width_label_mount_point);
+
+        if (disk_display->label_disk != NULL)
+        {
+            gtk_label_set_width_chars(GTK_LABEL(disk_display->label_disk), max_width_label_disk);
+        }
     }
 
 }
@@ -158,42 +179,53 @@ disk_display_new (t_disk *disk, t_mounter *mounter)
 
         if (mounter->trim_devicenames)
         {
-          if (g_str_has_prefix(disk->device, "/dev/mapper/"))
-            format_LVM_name (disk->device_short, &formatted_diskname);
-          else
-            formatted_diskname = g_strdup(disk->device_short);
+            if (g_str_has_prefix(disk->device, "/dev/mapper/"))
+                format_LVM_name (disk->device_short, &formatted_diskname);
+            else
+                formatted_diskname = g_strdup(disk->device_short);
         }
         else
         {
-          if (g_str_has_prefix(disk->device, "/dev/mapper/"))
-            format_LVM_name (disk->device, &formatted_diskname);
-          else
-            formatted_diskname = g_strdup(disk->device);
+            if (g_str_has_prefix(disk->device, "/dev/mapper/"))
+                format_LVM_name (disk->device, &formatted_diskname);
+            else
+                formatted_diskname = g_strdup(disk->device);
         }
 
         if (mounter->exclude_devicenames)
-            dd->label_disk = gtk_label_new (disk->mount_point);
+        {
+            dd->label_disk = NULL;
+            dd->label_mount_arrow = NULL;
+        }
         else
-            dd->label_disk = gtk_label_new (g_strconcat(formatted_diskname, _(" -> "),
-                                        disk->mount_point, NULL));
+        {
+            dd->label_disk = gtk_label_new (formatted_diskname);
+            dd->label_mount_arrow = gtk_label_new (_(" -> "));
 
+            /*change to uniform label size*/
+            /*gtk_label_set_width_chars(GTK_LABEL(dd->label_disk), 32); */
+            gtk_label_set_xalign(GTK_LABEL(dd->label_disk), 1.0);
+            gtk_widget_set_valign(dd->label_disk,GTK_ALIGN_CENTER);
+            gtk_widget_set_valign(dd->label_mount_arrow,GTK_ALIGN_CENTER);
+
+            gtk_box_pack_start(GTK_BOX(dd->hbox),dd->label_disk,FALSE,TRUE,0);
+            gtk_box_pack_start(GTK_BOX(dd->hbox),dd->label_mount_arrow,FALSE,TRUE,0);
+        }
         g_free (formatted_diskname);
 
-        /*change to uniform label size*/
-        /*gtk_label_set_width_chars(GTK_LABEL(dd->label_disk), 32); */
-        /* gtk_label_set_justify(GTK_LABEL(dd->label_disk),GTK_JUSTIFY_LEFT); */
-        gtk_widget_set_valign(dd->label_disk,GTK_ALIGN_CENTER);
-        gtk_box_pack_start(GTK_BOX(dd->hbox),dd->label_disk,FALSE,TRUE,0);
+        dd->label_mount_point = gtk_label_new (disk->mount_point);
+        gtk_label_set_xalign(GTK_LABEL(dd->label_mount_point), 0.0);
+        gtk_widget_set_valign(dd->label_mount_point,GTK_ALIGN_CENTER);
+        gtk_box_pack_start(GTK_BOX(dd->hbox),dd->label_mount_point,FALSE,TRUE,0);
 
         dd->label_mount_info = gtk_label_new("");
         /*change to uniform label size*/
         /* gtk_label_set_width_chars(GTK_LABEL(dd->label_mount_info),25); */
         gtk_label_set_use_markup(GTK_LABEL(dd->label_mount_info),TRUE);
-        /* gtk_label_set_justify (GTK_LABEL(dd->label_mount_info),
-                               GTK_JUSTIFY_RIGHT); */
-        gtk_widget_set_halign((dd->label_mount_info),GTK_ALIGN_END);
+        gtk_label_set_xalign (GTK_LABEL(dd->label_mount_info),
+                               1.0);
         gtk_widget_set_valign((dd->label_mount_info),GTK_ALIGN_CENTER);
-        gtk_box_pack_start(GTK_BOX(dd->hbox),dd->label_mount_info,TRUE,TRUE,0);
+        gtk_box_pack_start(GTK_BOX(dd->hbox),dd->label_mount_info,FALSE,TRUE,0);
 
         dd->progress_bar = gtk_progress_bar_new();
         gtk_box_pack_start(GTK_BOX(dd->hbox),dd->progress_bar,TRUE,TRUE,0);
@@ -237,6 +269,10 @@ disk_display_refresh (t_disk_display * disk_display)
             gtk_progress_bar_set_fraction (
                      GTK_PROGRESS_BAR(disk_display->progress_bar),
                      ((gdouble)mount_info->percent / 100) );
+            gtk_progress_bar_set_show_text (
+                     GTK_PROGRESS_BAR(disk_display->progress_bar),
+                     TRUE);
+
             gtk_progress_bar_set_text (
                      GTK_PROGRESS_BAR(disk_display->progress_bar),
                      g_strdup_printf ("%d%%",mount_info->percent));
@@ -290,6 +326,7 @@ mounter_data_new (t_mounter *mt)
     unsigned int i;
     t_disk * disk;
     t_disk_display * disk_display;
+    GtkWidget * title_menu_item, *title_label;
     GPtrArray *array =  NULL, *disk_displays = NULL;
     char *dev_mp; /* device or mountpoint */
 
@@ -317,6 +354,14 @@ mounter_data_new (t_mounter *mt)
 
     /* menu with menu_item */
     mt->menu = gtk_menu_new ();
+
+    title_menu_item = gtk_menu_item_new();
+    title_label = gtk_label_new(NULL);
+    gtk_label_set_markup(GTK_LABEL(title_label), _("<b><i><span font_size=\"large\">Xfce 4 Mount Plugin – Devices and Mount Points</span></i></b>"));
+    gtk_container_add (GTK_CONTAINER(title_menu_item), title_label);
+    gtk_menu_shell_append (GTK_MENU_SHELL(mt->menu),
+                           title_menu_item);
+
     disk_displays =g_ptr_array_new();
     for (i=0; i < mt->pdisks->len; i++)
     {
@@ -331,6 +376,7 @@ mounter_data_new (t_mounter *mt)
         gtk_menu_shell_append (GTK_MENU_SHELL(mt->menu),
                                disk_display->menu_item);
     }
+
     gtk_widget_show_all(mt->menu);
 
     disk_display_set_sizes(disk_displays);
@@ -357,22 +403,22 @@ static gboolean
 on_button_press (GtkWidget *widget, GdkEventButton *eventButton, t_mounter *mounter)
 {
     gboolean result = FALSE;
-#if GTK_CHECK_VERSION (3,22,0)
-    GdkEvent event;
-#endif
+
     TRACE ("enters on_button_press");
     if (eventButton != NULL)
     {
         if (mounter != NULL && eventButton->button == 1) /* left click only */
         {
             mounter_refresh (mounter); /* refreshs infos regarding mounts data */
-#if GTK_CHECK_VERSION (3,22,0)
-            event.button = *eventButton;
-            gtk_menu_popup_at_widget (GTK_MENU(mounter->menu), mounter->button, GDK_GRAVITY_CENTER, GDK_GRAVITY_CENTER, &event);
+#if LIBXFCE4PANEL_CHECK_VERSION (4, 17 ,2)
+            xfce_panel_plugin_popup_menu (mounter->plugin, GTK_MENU (mounter->menu),
+                                          mounter->button, (GdkEvent *) eventButton);
 #else
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
             gtk_menu_popup (GTK_MENU(mounter->menu), NULL, NULL,
                             xfce_panel_plugin_position_menu, mounter->plugin,
                             0, eventButton->time);
+G_GNUC_END_IGNORE_DEPRECATIONS
 #endif
             result = TRUE;
         }
@@ -523,7 +569,7 @@ create_mounter_control (XfcePanelPlugin *plugin)
     g_assert (mounter->icon!=NULL);
 
     mounter->button = gtk_button_new ();
-    mounter->image = xfce_panel_image_new_from_source (mounter->icon);
+    mounter->image = gtk_image_new ();
     gtk_widget_show(mounter->image);
     gtk_container_add (GTK_CONTAINER(mounter->button), mounter->image);
     gtk_button_set_relief (GTK_BUTTON(mounter->button), GTK_RELIEF_NONE);
@@ -611,10 +657,7 @@ mounter_apply_options (t_mounter_dialog *md)
                    "%s/icons/hicolor/scalable/apps/xfce-mount.svg",
                    PACKAGE_DATA_DIR );
 
-    gtk_container_remove(GTK_CONTAINER(mt->button), mt->image);
-    mt->image = xfce_panel_image_new_from_source (mt->icon);
-    gtk_widget_show(mt->image);
-    gtk_container_add (GTK_CONTAINER(mt->button), mt->image);
+    mounter_set_size (mt->plugin, xfce_panel_plugin_get_size (mt->plugin), mt);
 
     TRACE ("leaves mounter_apply_options");
 }
@@ -1036,24 +1079,19 @@ mounter_create_options (XfcePanelPlugin *plugin, t_mounter *mt)
 static void
 mounter_show_about(XfcePanelPlugin *plugin, t_mounter *mt)
 {
-   GdkPixbuf *icon;
    const gchar *auth[] = { "Jean-Baptiste Dulong",
                            "Fabian Nowak <timystery@arcor.de>",
                            "Landry Breuil <landry@xfce.org>", NULL };
-   icon = xfce_panel_pixbuf_from_source("drive-harddisk", NULL, 32);
    gtk_show_about_dialog(NULL,
-      "logo", icon,
+      "logo-icon-name", "drive-harddisk",
       "license", xfce_get_license_text (XFCE_LICENSE_TEXT_GPL),
       "version", PACKAGE_VERSION,
       "program-name", PACKAGE_NAME,
       "comments", _("Show partitions/devices and allow to mount/unmount them"),
-      "website", "http://goodies.xfce.org/projects/panel-plugins/xfce4-mount-plugin",
-      "copyright", _("Copyright (c) 2005-2016\n"),
+      "website", "https://docs.xfce.org/panel-plugins/xfce4-mount-plugin",
+      "copyright", "Copyright \302\251 2005-2023 The Xfce development team",
       "authors", auth, NULL);
   // TODO: add translators.
-
-   if(icon)
-      g_object_unref(G_OBJECT(icon));
 }
 
 static void
@@ -1065,17 +1103,17 @@ mount_construct (XfcePanelPlugin *plugin)
 
     mounter = create_mounter_control (plugin);
 
-#ifdef HAS_PANEL_49
     if (xfce_panel_plugin_get_mode(plugin) == XFCE_PANEL_PLUGIN_MODE_DESKBAR)
         xfce_panel_plugin_set_small(plugin, FALSE);
     else
         xfce_panel_plugin_set_small (plugin, TRUE);
-#endif
 
     g_signal_connect (plugin, "free-data", G_CALLBACK (mounter_free), mounter);
 
-    g_signal_connect (plugin, "save", G_CALLBACK (mounter_write_config),
-                      mounter);
+/* superfluous, as potential race condition with too fast shutdown of the panel exists;
+ * and unnecessary because stuff is saved when closing the settings dialog */
+/*    g_signal_connect (plugin, "save", G_CALLBACK (mounter_write_config),
+                      mounter);*/
 
     xfce_panel_plugin_menu_show_configure (plugin);
     g_signal_connect (plugin, "configure-plugin",
@@ -1084,6 +1122,7 @@ mount_construct (XfcePanelPlugin *plugin)
     xfce_panel_plugin_menu_show_about(plugin);
     g_signal_connect (plugin, "about", G_CALLBACK (mounter_show_about), mounter);
 
+    mounter_set_size (plugin, xfce_panel_plugin_get_size (plugin), mounter);
     g_signal_connect (plugin, "size-changed", G_CALLBACK (mounter_set_size),
                          mounter);
 
